#!/usr/bin/env python
# *********************************************************************
# * Copyright (C) 2012 Luca Baldini (luca.baldini@pi.infn.it)         *
# *                                                                   *
# * For the license terms see the file LICENCE, distributed           *
# * along with this software.                                         *
# *********************************************************************
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os
import re
import sys
import glob

from plasduino.__plasduino__ import PLASDUINO_ARDUINO
from plasduino.__logging__ import logger, abort



HEADER_FILES = glob.glob(os.path.join(PLASDUINO_ARDUINO, '*.h'))
ARDUINO_BOARD_FILE_NAME = 'boards.txt'
SUPPORTED_BOARDS = ['uno', 'diecimila']

PLATFORM = os.name

""" Platform specific path to the arduino configuration file
(assuming the arduino software is installed).

TODO: need to read the installation from the system-wide configuration file.
"""
if PLATFORM == 'posix':
    ARDUINO_HOME = os.environ.get('ARDUINO_HOME', '/usr/share/arduino/')
elif PLATFORM == 'nt':
    ARDUINO_HOME = os.environ.get('ARDUINO_HOME', None)
else:
    abort('Platform %s not supported (yet)' % PLATFORM)


ARDUINO_BOARD_FILE_PATH = os.path.join(PLASDUINO_ARDUINO,
                                       ARDUINO_BOARD_FILE_NAME)

def pulseDTR(port):
    """ Assert the DTR for 0.5 s on the specified port and then de-assert it.
    """
    import serial
    import time
    logger.info('Pulsing serial control line...')
    ser = serial.Serial(port)
    ser.setDTR(1)
    time.sleep(0.5)
    ser.setDTR(0)
    ser.close()

def getHexFileName(sketchName, board):
    """ Return the file name for the binary file of the sketch compile for a
    given board.
    """
    return '%s-%s.hex' % (sketchName, board)

def searchArduinoBoardFile(pattern):
    """ Dig into the ARDUINO_BOARD_FILE_PATH file and search for matches
    to a given regular expression.

    At this point we just abort if the regular expression has no match in the
    file.
    """
    ptn = re.compile(pattern)
    for line in open(ARDUINO_BOARD_FILE_PATH):
        result = ptn.findall(line)
        if result:
            return result[0]
    sys.exit('No luck with pattern "%s" in file %s. Abort.' %\
                 (pattern, ARDUINO_BOARD_FILE_PATH))

def listArduinoBoards():
    boards = []
    for line in open(ARDUINO_BOARD_FILE_PATH):
        if '.name' in line:
            boards.append(line.split('.name')[0])
    return boards

def getUploadProtocol(model):
    """ Retrieve the upload protocol for a give board.
    """
    pattern = r'^%s\.upload\.protocol=(.*)' % model
    protocol = searchArduinoBoardFile(pattern)
    if protocol == 'stk500':
        UPLOAD_PROTOCOL = 'stk500v1'
    return protocol

def getUploadSpeed(model):
    """ Retrieve the upload speed for a given board.
    """
    pattern = r'^%s\.upload\.speed=(.*)' % model
    return searchArduinoBoardFile(pattern)

def getMCU(model):
    """ Return the name of the MicroController Unit onboard.
    """
    pattern = r'^%s\.build\.mcu=(.*)' % model
    return searchArduinoBoardFile(pattern)

def getCPUFrequency(model):
    """ Return the clock frequency of the microcontroller.
    """
    pattern = r'^%s\.build\.f_cpu=(.*)' % model
    return searchArduinoBoardFile(pattern)

def getMaximumUploadSize(model):
    """ Return the maximum upload size.
    """
    pattern = r'^%s\.upload.maximum_size=(.*)' % model
    return searchArduinoBoardFile(pattern)

def getSketchInfo(sketchName):
    """ Open the sketch (.ino) file and figure out the sketch id and version.
    """
    from plasduino.arduino.sketches_h import *
    fileName = '%s.ino' % sketchName
    filePath = os.path.join(PLASDUINO_ARDUINO, sketchName, fileName)
    content = open(filePath, 'r').read()
    try:
        sketchId = re.search('(?<=SKETCH_ID).+', content).group(0)
        sketchId = eval(sketchId)
    except:
        logger.warn('Could not retrieve sketch ID for %s.' % sketchName)
        sketchId = None
    try:
        sketchVersion =\
            re.search('(?<=SKETCH_VERSION).+', content).group(0).strip()
        sketchVersion = int(sketchVersion)
    except:
        logger.warn('Could not retrieve sketch version for %s.' % sketchName)
        sketchVersion = None
    return sketchId, sketchVersion

def listSketches():
    """ List all the sketches available for upload.
    """
    sketchList = []
    for folderPath in glob.glob(os.path.join(PLASDUINO_ARDUINO, '*')):
        if os.path.isdir(folderPath):
            sketchName = os.path.basename(folderPath)
            if glob.glob(os.path.join(folderPath, '*.hex')):
                sketchList.append(sketchName)
    sketchList.sort()  
    return sketchList



if __name__ == '__main__':
    for item in dir():
        if item.isupper():
            print('%s = %s' % (item, eval(item)))
    for sketchName in listSketches():
        try:
            print('%s version: %s' % (sketchName, getSketchInfo(sketchName)))
        except:
            print('*** Cannot determine info for %s.' % sketchName)
    print('Arduino boards: %s' % listArduinoBoards())
